;___________________
;	QUARZO DA 4 MHZ	|
;___________________|

	LIST P=16F84A
;___________________________________
;	INDIRIZZI REGISTRI E BIT VARI	|
;___________________________________|
TIMER	EQU		01h
INTCON	EQU		0Bh
OPTIO	EQU		01h
PORTA	EQU		05h
PORTB	EQU		06h
TOIE	EQU		05h
TOIF	EQU		02h
GIE		EQU		07h
STATUS	EQU		03h
ZERO	EQU		02h
CARRY	EQU		00h
BMENU	EQU		00h
BSET	EQU		01h
IF_ON_OFF	EQU	06h		;	BIT DEL REGISTRO CONF CHE INDICA SE SI HA CALCOLATO LA DIFFERENZA

;___________________________
;	LINEE DI PORTB PER LCD	|
;___________________________|

;	>>  USO DEI BIT DI PORTB  <<
;					 ______________________________________________
;	uso				|	X |	 X |  RS |   E |  D7 |  D6 |  D5 |  D4 |
;	Nbit			|___7_|__6_|___5_|___4_|___3_|___2_|___1_|___0_|

RS		EQU		05h
EN		EQU		04h

;	>>  USO DEI BIT DI PORTA  <<
;					 ______________________________________________
;	uso				|	X |	 X |   X |  RF |   X |   X | SET | MENU|
;	Nbit			|___7_|__6_|___5_|___4_|___3_|___2_|___1_|___0_|

;	>>  REGISTRO CONF  <<
;					 ________________________________________________________
;	uso				|	X |	 IF_ON_OFF |   X | RF IN |   X |   X | P1  | P0  |
;	Nbit			|___7_|_____6______|___5_|___4___|___3_|__2__|__1__|__0__|

;___________________
;	REGISTRI RAM	|
;___________________|
DELAY_H	EQU		0Ch		;	BYTE ALTO DEL CICLO DI RITARDO
DELAY_L	EQU		0Dh		;	BYTE BASSO DEL CICLO DI RITARDO

DE_KILO	EQU		0Eh		;	DECINE DI KILOHERTZ
CE_KILO	EQU		0Fh		;	CENTINAIA DI KILOHERTZ
UNITA	EQU		10h		;	UNIT DI MHz
DECINE	EQU		11h		;	DECINE DI MHz
CENT	EQU		12h		;	CENTINAIA DI MHz
GIGA	EQU		13h		;	GIGAHERTZ
RESTO	EQU		14h		;	VALORE DEL TIMER AL TERMINE DEL Tm

NIB_L	EQU		15h		;	NIBBLE BASSO DA INVIARE AL DISPLAY
NIB_H	EQU		16h		;	NIBBLE ALTO DA INVIARE AL DISPLAY

CONF	EQU		17h		;	BYTE DI CONFIGURAZIONE ; IL BIT DI PESO PI ALTO VALE 1 SE  ATTIVATA LA IF

TEMP	EQU		18h		;	REGISTRO USATO PER VARIE TEMPORIZZAZIONI O CICLI

IF_HH	EQU		19h		;	CENTINAIA DI MEGAHERTZ DELLA if
IF_H	EQU		20h		;	DECINE DI MEGAHERTZ DELLA if
IF_L	EQU		21h		;	UNITA DI MEGAHERTZ DELLA if
IF_D	EQU		22h		;	CENTINAIA DI KILOHERTZ DELLA if

CIFRA_IF	EQU	23h		;	INDICA LA CIFRA DELLA FREQUENZA INTERMEDIA SELEZIONATA; VALE 0 SE  SELEZIONATA LA CIFRA DI PESO MAGGIORE


		ORG	0
		GOTO	INIZIO	;	SALTA LA ROUTINE DI INTERRUPT

;___________________________________
;	ROUTINE DI INTERRUPT DEL TIMER	|
;___________________________________|
		ORG	4

I_CE_KILO
		INCF	CE_KILO,1

;	VERIFICA SE C' RIPORTO DA SOMMARE ALLE CIFRE DI PESO SUPERIORE
T_KILO	MOVLW	.10
		XORWF	CE_KILO,0
		BTFSS	STATUS,ZERO
		GOTO	ESCI
T_UNI	CLRF	CE_KILO
		INCF	UNITA,1
		MOVLW	.10
		XORWF	UNITA,0
		BTFSS	STATUS,ZERO
		GOTO	ESCI		
T_DEC	CLRF	UNITA
		INCF	DECINE,1
		MOVLW	.10
		XORWF	DECINE,0
		BTFSS	STATUS,ZERO
		GOTO	ESCI	
T_CENT	CLRF	DECINE
		INCF	CENT,1
		MOVLW	.10
		XORWF	CENT,0
		BTFSS	STATUS,ZERO
		GOTO	ESCI	
T_GIGA	CLRF	CENT
		INCF	GIGA,1
ESCI	BCF		INTCON,TOIF	;	AZZERA FLAG DI AVVENUTO INTERRUPT
		RETFIE				;	ED ESCE DALL'INTERRUPT
;___________________
;	FINE INTERRUPT	|
;___________________|

INIZIO
		CALL	R20m		;	RITARDO INIZIALE DI 20 mS PER LCD
;_______________________
;	CONFIGURAZIONE PIC	|
;_______________________|
		BSF		STATUS,5	;	=> BANK 1 IN USO
		MOVLW	0
		MOVWF	06h			;	PORTB COME USCITA
		MOVLW	0FFh
		MOVWF	05h			;	PORTA COME INGRESSO
		BSF		INTCON,GIE	;	ABILITAZIONE GENERALE DEGLI INTERRUPT
		BSF		OPTIO,5		;	SORGENTE DI CLOCK SU RA4		
		BCF		STATUS,5	;	=> BANK 0 IN USO

;___________________________
;	IMPOSTO IL BUS A 4 BIT	|
;___________________________|
		MOVLW	.3			; FUNCTION SET
		MOVWF	PORTB
		CALL	E
		CALL	R5m

		MOVLW	.3			; FUNCTION SET
		MOVWF	PORTB
		CALL	E
		CALL	R200u

		MOVLW	.3			; FUNCTION SET
		MOVWF	PORTB
		CALL	E
		CALL	R200u

		MOVLW	b'00000010'	; FUNCTION SET
		MOVWF	PORTB
		CALL	E
		CALL	R5m

;___________________________
; 	IMPOSTO USO DI 2 RIGHE	|
;___________________________|
		MOVLW	b'00101000'
		CALL	INVIO
;_______________________
;	ACCENDO IL DISPLAY	|
;_______________________|
		MOVLW	b'00001100'	;	COMANDO DISPLAY ON/OFF & CURSOR
		CALL	INVIO		;	IL DISPLAY  ORA ACCESO
		CALL	CLEAR		;	PULISCE LCD E MANDA IL CURSORE ALL'INIZIO
;___________________________
;	SELEZIONE MODALIT DATI	|
;___________________________|
		BSF		PORTB,5
		CALL	R5m
;_______________________
;	SCRITTA INIZIALE	|
;_______________________|
		MOVLW	' '
		CALL	INVIO
		MOVLW	'F'
		CALL	INVIO
		MOVLW	'r'
		CALL	INVIO
		MOVLW	'e'
		CALL	INVIO
		MOVLW	'q'
		CALL	INVIO
		MOVLW	'u'
		CALL	INVIO
		MOVLW	'e'
		CALL	INVIO
		MOVLW	'n'
		CALL	INVIO
		MOVLW	'z'
		CALL	INVIO
		MOVLW	'i'
		CALL	INVIO
		MOVLW	'm'
		CALL	INVIO
		MOVLW	'e'
		CALL	INVIO
		MOVLW	't'
		CALL	INVIO
		MOVLW	'r'
		CALL	INVIO
		MOVLW	'o'
		CALL	INVIO
		CALL	R5s

;___________________
;	SELEZIONE BANDA	|
;___________________|

MENU	CLRF	CONF
		CALL	IF_CLEAR
		CALL	CLEAR
		MOVLW	'S'
		CALL	INVIO
		MOVLW	'e'
		CALL	INVIO
		MOVLW	'l'
		CALL	INVIO
		MOVLW	'e'
		CALL	INVIO
		MOVLW	'z'
		CALL	INVIO
		MOVLW	'.'
		CALL	INVIO
		MOVLW	' '
		CALL	INVIO
		MOVLW	'b'
		CALL	INVIO
		MOVLW	'a'
		CALL	INVIO
		MOVLW	'n'
		CALL	INVIO
		MOVLW	'd'
		CALL	INVIO
		MOVLW	'a'
		CALL	INVIO
		CALL	RIGA2
		MOVLW	.12
		CALL	DX
		CALL	LCD0
;_______________________________
;	CONTROLLO STATO PULSANTI	|
;_______________________________|
CHK_SET	BTFSS	PORTA,BSET
		GOTO	CHK_MEN
		BTFSC	PORTA,BSET
		GOTO	$-1
		BTFSS	CONF,0
		GOTO	MF
		GOTO	HF
CHK_MEN	BTFSS	PORTA,BMENU
		GOTO	CHK_SET
		BTFSC	PORTA,BMENU
		GOTO	$-1
		INCF	CONF,1

		BTFSS	CONF,1
		GOTO	$+3
		BCF		CONF,0
		BCF		CONF,1
		BTFSS	CONF,0
		GOTO	LCD0
		GOTO	LCD1
;_______________________
;	MEDIE FREQUENZE MF	|
;_______________________|

MF		CALL	LCD_MF

TmMF	;	TEMPO DI MISURA PARI A 2,56 s
		CALL	NUOVA

		MOVLW	.13
		MOVWF	TEMP
		MOVLW	.236
		MOVWF	DELAY_H		;	CARICA 83 IN DELAY_H
		MOVLW	.92
		MOVWF	DELAY_L		;	CARICA 256 IN DELAY_L

		BSF		INTCON,TOIE	;	ABILITA L'INTERRUPT SU OVERFLOW DEL TIMER

		CALL	LOOP		;	DA QUI INIZIA LA MISURA
		DECFSZ	TEMP,1
		GOTO	$-2
		BCF		INTCON,TOIE	;	DISABILITA TEMPORANEAMENTE L'INTERRUPT SU OVERFLOW DEL TIMER
		MOVF	TIMER,0		;	COPIA TIMER IN W
		MOVWF	RESTO		;	E POI LO SALVA IN "RESTO"
		CALL	KHz			;	MISURA LE DECINE DI KILOHERTZ
		MOVWF	DE_KILO
;___________________________
;	VISUALIZZAZIONE SU LCD	|
;___________________________|
		CALL	ELABOR
		MOVLW	.7
		CALL	SX
		MOVF	GIGA,0
		CALL	INVIO
		MOVF	CENT,0
		CALL	INVIO
		MOVF	DECINE,0
		CALL	INVIO
		MOVF	UNITA,0
		CALL	INVIO
		MOVLW	.1
		CALL	DX
		MOVF	CE_KILO,0
		CALL	INVIO
		MOVF	DE_KILO,0
		CALL	INVIO

		BTFSC	PORTA,BMENU
		GOTO	MENU
		GOTO	TmMF			;	RIPARTE CON LA NUOVA MISURA
;___________________________
;	ALTE FREQUENZE	HF/VHF	|
;___________________________|
HF		CALL	LCD_HF

TmHF	;	TEMPO DI MISURA PARI A 1,31072 s (AGGIORNATO A 1,28915 s)
		CALL	NUOVA
		MOVLW	.8
		MOVWF	DELAY_L
		MOVLW	.136
		MOVWF	DELAY_H
		MOVLW	.7
		MOVWF	TEMP
		BSF		INTCON,TOIE	;	ABILITA L'INTERRUPT SU OVERFLOW DEL TIMER

CICLO0	CALL	LOOP
		DECFSZ	TEMP,15
		GOTO	RIC
;	FINITO IL PERIODO DI MISURA

		BCF		INTCON,TOIE	;	DISABILITA TEMPORANEAMENTE L'INTERRUPT SU OVERFLOW DEL TIMER
		MOVF	TIMER,0		;	COPIA TIMER IN W
		MOVWF	RESTO		;	E POI LO SALVA IN "RESTO"
		CALL	KHz			;	MISURA LE DECINE DI KILOHERTZ
		MOVWF	DE_KILO
;___________________________
;	VISUALIZZAZIONE SU LCD	|
;___________________________|
		BTFSC	CONF,IF_ON_OFF
		CALL	SOTTRAI_IF	;	SOTTRAI LA IF SE  ATTIVATA LA MODALIT
		CALL	ELABOR

		MOVLW	.6
		CALL	SX
		MOVF	CENT,0
		CALL	INVIO
		MOVF	DECINE,0
		CALL	INVIO
		MOVF	UNITA,0
		CALL	INVIO
		MOVLW	.1
		CALL	DX
		MOVF	CE_KILO,0
		CALL	INVIO
		MOVF	DE_KILO,0
		CALL	INVIO

		BTFSC	PORTA,BSET
		CALL	IMPOSTA_IF

		BTFSC	PORTA,BMENU		;	SE IL PULSANTE MENU  PREMUTO ALLORA RITORNA AL MENU INIZIALE
		GOTO	MENU			;	ALTRIMENTI ESEGUE UNA NUOVA MISURA CON LA STESSA PORTATA
		GOTO	TmHF


;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;+++++++++++++																	++++++++++++
;+++++++++++++				FINE PROGRAMMA PRINCIPALE							++++++++++++
;+++++++++++++																	++++++++++++
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++



;	 ///////////////////////////////|||||||||||||||||||\\\\\\\\\\\\\\\
;	|||||||||||||||||||  TEMPORIZZAZIONI	   ||||||||||||||||||||||||
;	 \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\|||||||||||||||||||///////////////
;_______________________
;	RITARDO GENERALE	|
;_______________________|
LOOP	DECFSZ	DELAY_L
		GOTO	LOOP
		DECFSZ	DELAY_H
		GOTO	RIC_L
		MOVLW	.255
		MOVWF	DELAY_L
ULTIMO	DECFSZ	DELAY_L
		GOTO	ULTIMO
		RETURN
RIC_L	MOVLW	.255
		MOVWF	DELAY_L
		GOTO	LOOP
;_______________________________________________________
;	RICARICA I REGISTRI "DELAY_H" E "DELAY_L" CON 256	|
;_______________________________________________________|
RIC		MOVLW	0FFh
		MOVWF	DELAY_H
		MOVWF	DELAY_L
		GOTO	CICLO0
;___________________
;	RITARDI VARI	|
;___________________|
R200u	MOVLW	.1
		MOVWF	DELAY_H
		MOVLW	.100
		MOVWF	DELAY_L
		CALL	LOOP
		RETURN
R5m		MOVLW	.32
		MOVWF	DELAY_H
		MOVLW	.255
		MOVWF	DELAY_L
		CALL	LOOP
		RETURN
R20m	MOVLW	.130
		MOVWF	DELAY_H
		MOVLW	.255
		MOVWF	DELAY_L
		CALL	LOOP
		RETURN
R5s		MOVLW	.10
		MOVWF	TEMP
		MOVLW	.255
		MOVWF	DELAY_L
		MOVWF	DELAY_H
SEC5	DECFSZ	TEMP,1
		GOTO	MENO
		RETURN
MENO	CALL	LOOP
		GOTO	SEC5

;	 ///////////////////////////////|||||||||||||||||||\\\\\\\\\\\\\\\
;	|||||||||||||||||||  ROUTINE DEL DISPLAY   ||||||||||||||||||||||||
;	 \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\|||||||||||||||||||///////////////
;___________
;	ENABLE	|
;___________|
E		BSF		PORTB,EN
		NOP
		NOP
		BCF		PORTB,EN
		RETURN
;___________________
;	AZZERA DISPLAY	|
;___________________|
CLEAR	BCF		PORTB,RS
		MOVLW	.1
		CALL	INVIO
		BSF		PORTB,RS
		RETURN
;___________________________________________________________________
;	SPOSTA IL CURSORE A SX DI n SPAZI PARI AL VALORE CARICATO IN W	|
;___________________________________________________________________|
SX		MOVWF	TEMP
		BCF		PORTB,RS
		MOVLW	b'00010000'
		CALL	INVIO
		DECFSZ	TEMP,1
		GOTO	$-3
		BSF		PORTB,RS
		RETURN
;___________________________________________________________________
;	SPOSTA IL CURSORE A DX DI n SPAZI PARI AL VALORE CARICATO IN W	|
;___________________________________________________________________|
DX		MOVWF	TEMP
		MOVF	TEMP,0
		XORLW	.0
		BTFSC	STATUS,ZERO
		RETURN
		BCF		PORTB,RS
		MOVLW	b'00010100'
		CALL	INVIO
		DECFSZ	TEMP,1
		GOTO	$-3
		BSF		PORTB,RS
		RETURN
;___________________________________________________________
;	SCRIVE n SPAZI SUL DISPLAY PARI AL VALORE CARICATO IN W	|
;___________________________________________________________|
SPAZIO	MOVWF	TEMP
		MOVLW	' '
		CALL	INVIO
		DECFSZ	TEMP,1
		GOTO	$-3
		RETURN
;___________________________________________________
;	PORTA IL CURSORE ALL'INIZIO DELLA SECONDA RIGA	|
;___________________________________________________|
RIGA2	BCF		PORTB,RS
		MOVLW	b'11000000'
		CALL	INVIO
		BSF		PORTB,RS
		RETURN

DX_UNO	MOVLW	.1
		CALL	DX
		RETURN

POSIZIONE_QUATTRO
		CALL	RIGA2
		MOVLW	.4
		CALL	DX
		RETURN
		
;_______________________
;	SCRITTE PER IL MENU	|
;_______________________|
LCD_MF	BSF		STATUS,5
		MOVLW	0FFh
		MOVWF	OPTIO
		BCF		STATUS,5
		CALL	CLEAR
		CALL	FREQUENZA
		MOVLW	' '
		CALL	INVIO
		MOVLW	'K'
		CALL	INVIO
		MOVLW	'H'
		CALL	INVIO
		MOVLW	'z'
		CALL	INVIO
		CALL	RIGA2
		MOVLW	.6
		CALL	SPAZIO
		MOVLW	','
		CALL	INVIO
		MOVLW	.2
		CALL	SPAZIO
		RETURN

LCD_HF	BSF		STATUS,5
		MOVLW	b'11110001'
		MOVWF	OPTIO		;	PRESCALER INTERNO AL PIC DIVIDE PER 4
		BCF		STATUS,5
		CALL	CLEAR
		CALL	FREQUENZA
		MOVLW	' '
		CALL	INVIO
		MOVLW	'M'
		CALL	INVIO
		MOVLW	'H'
		CALL	INVIO
		MOVLW	'z'
		CALL	INVIO
		CALL	RIGA2
		MOVLW	.5
		CALL	SPAZIO
		MOVLW	','
		CALL	INVIO
		MOVLW	.2
		CALL	SPAZIO
		RETURN

LISTA0	MOVLW	' '
		CALL	INVIO
		CALL	FRECCIA
		MOVLW	'M'
		CALL	INVIO
		MOVLW	'F'
		CALL	INVIO
		MOVLW	' '
		CALL	INVIO
		MOVLW	' '
		CALL	INVIO
		MOVLW	'H'
		CALL	INVIO
		MOVLW	'F'
		CALL	INVIO
		MOVLW	'/'
		CALL	INVIO
		MOVLW	'V'
		CALL	INVIO
		MOVLW	'H'
		CALL	INVIO
		MOVLW	'F'
		CALL	INVIO
		RETURN

LISTA1	MOVLW	' '
		CALL	INVIO
		MOVLW	' '
		CALL	INVIO
		MOVLW	'M'
		CALL	INVIO
		MOVLW	'F'
		CALL	INVIO
		MOVLW	' '
		CALL	INVIO
		CALL	FRECCIA
		MOVLW	'H'
		CALL	INVIO
		MOVLW	'F'
		CALL	INVIO
		MOVLW	'/'
		CALL	INVIO
		MOVLW	'V'
		CALL	INVIO
		MOVLW	'H'
		CALL	INVIO
		MOVLW	'F'
		CALL	INVIO
		RETURN

LCD0	MOVLW	.12
		CALL	SX
		CALL	LISTA0
		GOTO	CHK_SET

LCD1	MOVLW	.12
		CALL	SX
		CALL	LISTA1
		GOTO	CHK_SET

FRECCIA	MOVLW	b'01111110'	;	FRECCETTA
		CALL	INVIO
		RETURN

FREQUENZA
		MOVLW	'F'
		CALL	INVIO
		MOVLW	'r'
		CALL	INVIO
		MOVLW	'e'
		CALL	INVIO
		MOVLW	'q'
		CALL	INVIO
		MOVLW	'u'
		CALL	INVIO
		MOVLW	'e'
		CALL	INVIO
		MOVLW	'n'
		CALL	INVIO
		MOVLW	'z'
		CALL	INVIO
		MOVLW	'a'
		CALL	INVIO
		RETURN

;	 ///////////////////////////////|||||||||||||||||||\\\\\\\\\\\\\\\
;	|||||||||||||||||||  ELABORAZIONE DATI     ||||||||||||||||||||||||
;	 \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\|||||||||||||||||||///////////////
;___________
;	INVIO	|
;___________|
;	DIVIDE IL BYTE NEL NIBBLE ALTO E IN QUELLO BASSO, E INVIA PRIMA L'ALTO E POI IL BASSO
INVIO	MOVWF	NIB_L
		MOVF	NIB_L,0
		MOVWF	NIB_H
		SWAPF	NIB_H
		MOVLW	b'00001111'	;	MASCHERA PER NIB_H
		ANDWF	NIB_H,1
		MOVLW	b'00001111'	;	MASCHERA PER NIB_L
		ANDWF	NIB_L,1
		MOVLW	b'11110000'	;	MASCHERA PER PORT_B: AZZERA D7 D6 D5 E D4
		ANDWF	PORTB,1
		MOVF	NIB_H,0
		ADDWF	PORTB,1
		CALL	E
		BTFSS	PORTB,RS
		CALL	R5m			;	RITARDO PER I COMANDI
		CALL	R200u		;	RITARDO PER I DATI
;	NIBBLE ALTO FATTO
		MOVLW	b'11110000'	;	MASCHERA PER PORT_B: AZZERA D7 D6 D5 E D4
		ANDWF	PORTB,1
		MOVF	NIB_L,0
		ADDWF	PORTB,1
		CALL	E
		BTFSS	PORTB,RS
		CALL	R5m			;	RITARDO PER I COMANDI
		CALL	R200u		;	RITARDO PER I DATI
;	NIBBLE BASSO FATTO
		RETURN
;___________________________________
;	MISURA DELLE CENTINAIA DI KHz	|
;___________________________________|
KHz		MOVLW	.25
		ADDWF	RESTO,0
		BTFSC	STATUS,CARRY
		RETLW	.9
		MOVLW	.51
		ADDWF	RESTO,0
		BTFSC	STATUS,CARRY
		RETLW	.8
		MOVLW	.76
		ADDWF	RESTO,0
		BTFSC	STATUS,CARRY
		RETLW	.7
		MOVLW	.102
		ADDWF	RESTO,0
		BTFSC	STATUS,CARRY
		RETLW	.6
		MOVLW	.127
		ADDWF	RESTO,0
		BTFSC	STATUS,CARRY
		RETLW	.5
		MOVLW	.153
		ADDWF	RESTO,0
		BTFSC	STATUS,CARRY
		RETLW	.4
		MOVLW	.179
		ADDWF	RESTO,0
		BTFSC	STATUS,CARRY
		RETLW	.3
		MOVLW	.204
		ADDWF	RESTO,0
		BTFSC	STATUS,CARRY
		RETLW	.2
		MOVLW	.230
		ADDWF	RESTO,0
		BTFSC	STATUS,CARRY
		RETLW	.1
		RETLW	.0
;_______________________________________
;	ELABORAZIONE DATI PER INVIO SU LCD	|
;_______________________________________|
ELABOR	MOVLW	.48
		ADDWF	GIGA,1
		ADDWF	CENT,1
		ADDWF	DECINE,1
		ADDWF	UNITA,1
		ADDWF	CE_KILO,1
		ADDWF	DE_KILO,1
		RETURN
;_______________________________________________________
;	AZZERA I REGISTRI RELATIVI ALLA MISURA PRECEDENTE	|
;_______________________________________________________|
NUOVA	CLRF	DE_KILO
		CLRF	CE_KILO
		CLRF	UNITA
		CLRF	DECINE
		CLRF	CENT
		CLRF	GIGA
		CLRF	TIMER
		CLRF	RESTO
		RETURN

;	 ///////////////////////////////|||||||||||||||||||\\\\\\\\\\\\\\\
;	||||||||||||||  ROUTINE VARIE FREQUENZA INTERMEDIA     ||||||||||||
;	 \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\|||||||||||||||||||///////////////

;___________________________________________________
;	ROUTINE PER IMPOSTARE LA FREQUENZA INTERMEDIA	|
;___________________________________________________|
IMPOSTA_IF
		BTFSC	PORTA,BSET	;	VERIFICO CHE IL PULSANTE SIA STATO RILASCIATO
		GOTO	$-1
		BSF		CONF,IF_ON_OFF		;	ATTIVA LA SOTTRAZIONE DELLA IF
		CALL	CLEAR
		MOVLW	'I'
		CALL	INVIO
		MOVLW	'm'
		CALL	INVIO
		MOVLW	'p'
		CALL	INVIO
		MOVLW	'o'
		CALL	INVIO
		MOVLW	's'
		CALL	INVIO
		MOVLW	't'
		CALL	INVIO
		MOVLW	'a'
		CALL	INVIO
		MOVLW	' '
		CALL	INVIO
		MOVLW	'I'
		CALL	INVIO
		MOVLW	'F'
		CALL	INVIO
		MOVLW	'('
		CALL	INVIO
		MOVLW	'M'
		CALL	INVIO
		MOVLW	'H'
		CALL	INVIO
		MOVLW	'z'
		CALL	INVIO
		MOVLW	')'
		CALL	INVIO

		CALL	AGGIORNA_IF

		CALL	RIGA2
		BCF		PORTB,RS
		MOVLW	b'00001110'	;	ATTIVA UNDERSCORE CURSORE
		CALL	INVIO
		BSF		PORTB,RS	;	E TORNA IN MODALITA DATI

IF_SET	BTFSS	PORTA,BSET
		GOTO	IF_MENU
		BTFSC	PORTA,BSET
		GOTO	$-1

;	VEDO QUALE CIFRA DELLA FREQUENZA INTERMEDIA  SELEZIONATA E LA INCREMENTA
		MOVLW	.3
		XORWF	CIFRA_IF,0
		BTFSC	STATUS,ZERO
		INCF	IF_D,1
		MOVLW	.2
		XORWF	CIFRA_IF,0
		BTFSC	STATUS,ZERO
		INCF	IF_L
		MOVLW	.1
		XORWF	CIFRA_IF,0
		BTFSC	STATUS,ZERO
		INCF	IF_H
		MOVLW	.0
		XORWF	CIFRA_IF,0
		BTFSC	STATUS,ZERO
		INCF	IF_HH

;	CONTROLLO SE OGNI CIFRA SUPERA IL 10: IN TAL CASO LA RIPORTO A ZERO
		MOVLW	.10
		XORWF	IF_HH,0
		BTFSC	STATUS,ZERO
		CLRF	IF_HH
		MOVLW	.10
		XORWF	IF_L,0
		BTFSC	STATUS,ZERO
		CLRF	IF_L
		MOVLW	.10
		XORWF	IF_H,0
		BTFSC	STATUS,ZERO
		CLRF	IF_H
		MOVLW	.10
		XORWF	IF_D,0
		BTFSC	STATUS,ZERO
		CLRF	IF_D

		CALL	AGGIORNA_IF
		GOTO	IF_SET

IF_MENU	BTFSS	PORTA,BMENU
		GOTO	IF_SET
		BTFSC	PORTA,BMENU
		GOTO	$-1

;	SOMMA UNO A CIFRA_IF E CONTROLLA SE DEVE USCIRE DAL MEN
		INCF	CIFRA_IF,1
		MOVLW	.4
		XORWF	CIFRA_IF,0
		BTFSC	STATUS,ZERO
		GOTO	ESCI_IF
		CALL	DX_UNO
		MOVLW	.3
		XORWF	CIFRA_IF,0
		BTFSC	STATUS,ZERO
		CALL	DX_UNO
		GOTO	IF_SET

ESCI_IF	BTFSC	PORTA,BMENU
		GOTO	$-1
		BCF		PORTB,RS
		MOVLW	b'00001100'	;	DISATTIVA UNDERSCORE CURSORE
		CALL	INVIO
		BSF		PORTB,RS
		CALL	CLEAR
		CLRF	CIFRA_IF

;	DISABILITO LA SOTTRAZIONE SE LA IF VALE ZERO
		MOVLW	.0
		XORWF	IF_D,0
		BTFSS	STATUS,ZERO
		GOTO	OK_ESCI
		MOVLW	.0
		XORWF	IF_L,0
		BTFSS	STATUS,ZERO
		GOTO	OK_ESCI
		MOVLW	.0
		XORWF	IF_H,0
		BTFSS	STATUS,ZERO
		GOTO	OK_ESCI
		MOVLW	.0
		XORWF	IF_HH,0
		BTFSS	STATUS,ZERO
		GOTO	OK_ESCI
		BCF		CONF,IF_ON_OFF
OK_ESCI	CALL	LCD_HF
		RETURN				;	ESCE DAL MENU DELLA IF

AGGIORNA_IF		
		CALL	RIGA2	;	VA A INIZIO RIGA E SCRIVE LA IF
		MOVF	IF_HH,0
		ADDLW	.48
		CALL	INVIO
		MOVF	IF_H,0
		ADDLW	.48
		CALL	INVIO
		MOVF	IF_L,0
		ADDLW	.48
		CALL	INVIO
		MOVLW	','
		CALL	INVIO
		MOVF	IF_D,0
		ADDLW	.48
		CALL	INVIO
;	ADESSO POSTA IL PUNTATORE
		CALL	RIGA2	;	POI SI RIPORTA SULLA CIFRA DELLA IF SELEZIONATA PRIMA DELL'INCREMENTO DELLA STESSA
		MOVF	CIFRA_IF,0
		CALL	DX
		MOVLW	.3
		XORWF	CIFRA_IF,0
		BTFSC	STATUS,ZERO
		CALL	POSIZIONE_QUATTRO
		RETURN

IF_CLEAR
		CLRF	IF_D
		CLRF	IF_L
		CLRF	IF_H
		CLRF	IF_HH
		RETURN

;___________________________________________
;		SOTTRAZIONE FREQUENZA INTERMEDIA	|
;___________________________________________|
SOTTRAI_IF
;*************	SOTTRAGGO CENTINAIA DI KILOHERTZ	***********************************
		MOVLW	.16
		ADDWF	CE_KILO,1	;	CARICA LE UNIT NEL W SOMMATE DI 10
		MOVF	IF_D,0
		SUBWF	CE_KILO,1
;CONTROLLO SE <10
		BTFSC	CE_KILO,4
		GOTO	CE_KILO_MAGGIORE
		DECF	UNITA,1
		MOVLW	.6
		SUBWF	CE_KILO,1
		GOTO	AVANTI0
;CONTROLLO SE >=10
CE_KILO_MAGGIORE
		MOVLW	.16
		SUBWF	CE_KILO,1
;*****************	SOTTRAGGO UNIT		*************************************************
AVANTI0	MOVLW	.16
		ADDWF	UNITA,1	;	CARICA LE UNIT NEL W SOMMATE DI 10
		MOVF	IF_L,0
		SUBWF	UNITA,1
;CONTROLLO SE <10
		BTFSC	UNITA,4
		GOTO	UNI_MAGGIORE
		DECF	DECINE,1
		MOVLW	.6
		SUBWF	UNITA,1
		GOTO	AVANTI1
;CONTROLLO SE >=10
UNI_MAGGIORE
		MOVLW	.16
		SUBWF	UNITA,1
;*************	SOTTRAGGO DECINE	*****************************************************
AVANTI1	MOVLW	.16
		ADDWF	DECINE,1	;	CARICA LE DECINE NEL W SOMMATE DI 10
		MOVF	IF_H,0
		SUBWF	DECINE,1		;	SOTTRAE LA if E METTE IL RISULTATO IN W
;CONTROLLO SE <10
		BTFSC	DECINE,4
		GOTO	DECINE_MAGGIORE
		DECF	CENT,1
		MOVLW	.6
		SUBWF	DECINE,1
		GOTO	AVANTI2
;CONTROLLO SE >=10
DECINE_MAGGIORE
		MOVLW	.16
		SUBWF	DECINE,1
;*************	SOTTRAGGO CENTINAIA		**************************************************
AVANTI2	MOVLW	.16
		ADDWF	CENT,1	;	CARICA LE UNIT NEL W SOMMATE DI 10
		MOVF	IF_HH,0
		SUBWF	CENT,1
;CONTROLLO SE <10
		BTFSC	CENT,4
		GOTO	CENT_MAGGIORE
		DECF	GIGA,1
		MOVLW	.6
		SUBWF	CENT,1
		RETURN
;CONTROLLO SE >=10
CENT_MAGGIORE
		MOVLW	.16
		SUBWF	CENT,1
		RETURN


		END